from xml.etree import ElementTree as ETree
from EEETools.MainModules import Block, ArrayHandler


class Drawer(Block):

    # This class is a simple block subclass that will be used for the definition of support blocks. See the component
    # documentation for further explanations.
    #
    # For the initialization both the input_ID and the main block to which the drawer is connected as a support block
    # are required. Another optional argument has to be passed if the drawer is intended to be added to handle the
    # outputs of the main block

    def __init__(self, main_class: ArrayHandler, main_block: Block, is_input=True, allow_multiple_input=True):

        super().__init__(-100, main_class, is_support_block=True)

        self.index = float(main_block.ID) + (float(len(main_block.support_block)) + 1.) / 100.
        self.type = "drawer"
        self.name = "Support block of: " + str(main_block.name)
        self.main_block = main_block

        self.is_input = is_input
        self.connection_with_main = None

        self.input_mixer = None
        self.output_separator = None

        self.allow_multiple_input = allow_multiple_input

    def add_connection(self, new_connection, is_input, append_to_support_block=None):

        # The "add_connection" method has been overloaded because we want the drawer to have only 3 connection (1 input,
        # 1 output and the "connection_with_main") if the "allow_multiple_input" option is not on. This is needed
        # because of some issue with the EES code generation.
        #
        # The overloaded method calls the super method if the maximum number of connection has not been reached or if
        # the "allow_multiple_input" option is on. Otherwise it will add the connection to a support_block (it choose
        # the right ones between the "input_mixer" or "output_separator" blocks depending on whether the connection
        # to be added is an input or an output) and initialize the support block itself if needed, by calling
        # "__append_support_block".

        if is_input:

            n_connection = self.n_input

            if self.connection_with_main is not None and not self.is_input:
                n_connection -= 1

            support_block = self.input_mixer

        else:

            n_connection = self.n_output

            if self.connection_with_main is not None and self.is_input:
                n_connection -= 1

            support_block = self.output_separator

        if self.allow_multiple_input or n_connection < 1:

            super(Drawer, self).add_connection(new_connection, is_input, append_to_support_block)

        else:

            if support_block is None:
                support_block = self.__append_support_block(is_input)

            support_block.add_connection(new_connection, is_input)

    def remove_connection(self, deleted_conn):

        # The "remove_connection" method has been overloaded because we want the drawer to have only 3 connection (1
        # input, 1 output and the "connection_with_main") if the "allow_multiple_input" option is not on. This is
        # needed because of some issue with the EES code generation.
        #
        # This method calls the super method to remove the connection. Then it checks if the support block (if
        # present) is still needed and remove it otherwise

        super(Drawer, self).remove_connection(deleted_conn)
        self.__check_support_blocks()

    def is_ready_for_calculation(self):

        return_value = self.n_input >= 1 and self.n_output >= 1

        if self.has_support_block:

            for block in self.support_block:
                return_value = return_value and block.is_ready_for_calculation()

        return return_value

    def prepare_for_calculation(self):

        if self.has_support_block:

            for block in self.support_block:

                block.prepare_for_calculation()

        self.update_main_connection()

    def initialize_connection_list(self, input_list):

        for elem in input_list:

            new_conn = self.main_class.find_connection_by_index(abs(elem))

            if not new_conn is None:

                is_input = (elem > 0)
                self.add_connection(new_conn, is_input)

    def connection_is_in_connections_list(self, connection):

        for connection_list in [self.input_connections, self.output_connections]:

            if connection in connection_list:
                return True

        if self.has_support_block:

            for block in self.support_block:

                if block.connection_is_in_connections_list(connection):
                    return True

        return False

    def get_fluid_stream_connections(self):

        connection_list = super(Drawer, self).get_fluid_stream_connections()

        if self.has_support_block:
            for block in self.support_block:
                connection_list.extend(block.get_fluid_stream_connections())

        return connection_list

    def return_other_zone_connections(self, zone_type, input_connection):

        return self.main_block.return_other_zone_connections(zone_type, input_connection)

    def initialize_main_connection(self):

        # connect support block with main class
        if self.is_input:
            self.__add_connection_manually(self, self.main_block, is_connection_with_main=True)

        else:
            self.__add_connection_manually(self.main_block, self, is_connection_with_main=True)

    def update_main_connection(self):

        if self.connection_with_main is None:

            self.initialize_main_connection()
            self.connection_with_main.is_fluid_stream = False

        if self.is_input:
            self.connection_with_main.exergy_value += self.exergy_balance

        else:
            self.connection_with_main.exergy_value -= self.exergy_balance

    def __add_connection_manually(self, from_block, to_block, is_connection_with_main=False):

        new_connection = self.main_class.append_connection()
        new_connection.set_block(from_block, is_from_block=True)
        from_block.output_connections.append(new_connection)
        from_block.n_output += 1

        new_connection.set_block(to_block, is_from_block=False)
        to_block.input_connections.append(new_connection)
        to_block.n_input += 1

        if is_connection_with_main:
            self.connection_with_main = new_connection

    def __append_support_block(self, is_input=True):

        # This class append an input mixer or an output separator if needed. In fact, drawer class is designed to
        # accept only 3 connections: 1 input, 1 output and the "connection_with_main" because of some issues
        # regarding the EES code definition.
        #
        # In order to deal with such limitation the derawer class is designed so that it will automatically append an
        # input mixer or an output separator so that it can replicate a multiple connection behaviour.
        #
        # This method initializes the support block (it choose between "input_mixer" and "output_separator" according
        # to the "is_input" parameter). It then moves all the input (or output) connections except for the
        # "connection_with_main" to the new block and generate the connection between the new support block and the
        # drawer class.

        self.has_support_block = True
        modules_handler = self.main_class.modules_handler

        if is_input:

            block_subclass = get_drawer_sub_class("mixer", modules_handler)
            self.input_mixer = block_subclass(self.main_class, self, True)
            self.support_block.append(self.input_mixer)

            for connection in self.input_connections:
                if connection is not self.connection_with_main:
                    self.input_mixer.add_connection(connection, is_input=True)

            self.input_connections = list()
            self.n_input = 0

            self.input_mixer.initialize_main_connection()

            return self.input_mixer


        else:

            block_subclass = get_drawer_sub_class("separator", modules_handler)
            self.output_separator = block_subclass(self.main_class, self, False)
            self.support_block.append(self.output_separator)

            for connection in self.output_connections:
                if connection is not self.connection_with_main:
                    self.output_separator.add_connection(connection, is_input=False)

            self.output_connections = list()
            self.n_output = 0

            self.output_separator.initialize_main_connection()

            return self.output_separator

    def __check_support_blocks(self):

        has_support_block = False

        if self.input_mixer is not None:

            if not self.input_mixer.is_needed:

                self.input_mixer.disconnect()
                self.support_block.remove(self.input_mixer)
                self.input_mixer = None

            else:

                has_support_block = True

        if self.output_separator is not None:

            if not self.output_separator.is_needed:

                self.output_separator.disconnect()
                self.support_block.remove(self.output_separator)
                self.output_separator = None

            else:

                has_support_block = True

        self.has_support_block = has_support_block

    @property
    def is_connected(self):
        return not self.connection_with_main is None

    def __str__(self):

        self.name = "Support block of: " + str(self.main_block.name)
        return super(Drawer, self).__str__()

    def export_xml_connection_list(self) -> ETree.Element:
        pass

    def append_xml_connection_list(self, input_list: ETree.Element):
        pass

    @classmethod
    def return_EES_needed_index(cls):
        pass

    @classmethod
    def return_EES_base_equations(cls):
        pass


def get_drawer_sub_class(block_subclass_name, modules_handler):

    block_subclass = modules_handler.import_correct_sub_class(block_subclass_name)

    class SupportSeparator(block_subclass):

        def __init__(self, main_class: ArrayHandler, main_block: Block, is_input=None):

            super().__init__(-100, main_class)

            self.is_support_block = True
            self.main_block = main_block

            self.__define_is_input(is_input)

            self.type += "-support block"
            self.name = "Support block of: " + str(main_block.name)

            self.connection_with_main = None

        def initialize_main_connection(self):

            if self.is_input:

                self.connection_with_main = self.main_class.append_connection(from_block=self, to_block=self.main_block)

            else:

                self.connection_with_main = self.main_class.append_connection(from_block=self.main_block, to_block=self)

        def update_main_connection(self):

            if self.connection_with_main is None:
                self.initialize_main_connection()

            if self.is_input:
                self.connection_with_main.exergy_value += self.__return_exergy_balance()

            else:
                self.connection_with_main.exergy_value -= self.__return_exergy_balance()

        def disconnect(self):

            if self.is_input:
                connection_list = self.input_connections

            else:
                connection_list = self.output_connections

            if self.connection_with_main is not None:
                self.main_class.remove_connection(self.connection_with_main)

            for conn in connection_list:
                self.main_block.add_connection(conn, is_input=self.is_input)

        def prepare_for_calculation(self):

            self.update_main_connection()

        def is_ready_for_calculation(self):

            return self.n_input >= 1 and self.n_output >= 1

        def __return_exergy_balance(self):

            exergy_balance = 0

            for conn in self.input_connections:
                exergy_balance += conn.exergy_value

            for conn in self.output_connections:
                exergy_balance -= conn.exergy_value

            return exergy_balance

        def __define_is_input(self, is_input):

            if is_input is None:

                if self.type == "mixer":

                    self.is_input = True

                elif self.type == "separator":

                    self.is_input = False

                else:

                    self.is_input = False

            else:

                self.is_input = is_input

        @property
        def is_needed(self):

            if self.is_input:

                return self.n_input > 1

            else:

                return self.n_output > 1

        @property
        def is_connected(self):

            return not self.connection_with_main is None

    return SupportSeparator


def get_support_block_sub_class(block_subclass_name, modules_handler):

    block_subclass = modules_handler.import_correct_sub_class(block_subclass_name)

    class SupportSeparator(block_subclass):

        def __init__(self, main_class: ArrayHandler, main_block: Block, is_input=None):

            super().__init__(-100, main_class)

            self.is_support_block = True
            self.main_block = main_block

            self.__define_is_input(is_input)

            self.type += "-support block"
            self.name = "Support block of: " + str(main_block.name)

            self.connection_with_main = None

        def initialize_main_connection(self):

            if self.is_input:

                self.connection_with_main = self.main_class.append_connection(from_block=self, to_block=self.main_block)

            else:

                self.connection_with_main = self.main_class.append_connection(from_block=self.main_block, to_block=self)

        def update_main_connection(self):

            if self.connection_with_main is None:
                self.initialize_main_connection()

            if self.is_input:
                self.connection_with_main.exergy_value += self.__return_exergy_balance()

            else:
                self.connection_with_main.exergy_value -= self.__return_exergy_balance()

        def disconnect(self):

            if self.is_input:
                connection_list = self.input_connections

            else:
                connection_list = self.output_connections

            if self.connection_with_main is not None:
                self.main_class.remove_connection(self.connection_with_main)

            for conn in connection_list:
                self.main_block.add_connection(conn, is_input=self.is_input)

        def prepare_for_calculation(self):

            self.update_main_connection()

        def is_ready_for_calculation(self):

            return self.n_input >= 1 and self.n_output >= 1

        def __return_exergy_balance(self):

            exergy_balance = 0

            for conn in self.input_connections:
                exergy_balance += conn.exergy_value

            for conn in self.output_connections:
                exergy_balance -= conn.exergy_value

            return exergy_balance

        def __define_is_input(self, is_input):

            if is_input is None:

                if self.type == "mixer":

                    self.is_input = True

                elif self.type == "separator":

                    self.is_input = False

                else:

                    self.is_input = False

            else:

                self.is_input = is_input

        @property
        def is_needed(self):

            if self.is_input:

                return self.n_input > 1

            else:

                return self.n_output > 1

        @property
        def is_connected(self):

            return not self.connection_with_main is None

    return SupportSeparator


from EEETools.MainModules.main_module import Connection, ArrayHandler, Block
from EEETools.BlockSubClasses.generic import Generic


class ProductBlock(Generic):

    def __init__(self, inputID, main_class, base_block: Block):

        super().__init__(inputID, main_class)
        self.base_block = base_block

        self.contained_blocks = [base_block]
        self.contained_connection = list()

    def add_connection(self, new_connection, is_input, append_to_support_block=None):

        super(ProductBlock, self).add_connection(new_connection, is_input, append_to_support_block=append_to_support_block)

        if not is_input:

            self.contained_connection.append(new_connection)

    def append_output_cost(self, defined_steam_cost):

        self.output_cost = defined_steam_cost
        self.base_block.output_cost = defined_steam_cost

        for outConn in self.contained_connection:

            if outConn.is_loss:
                outConn.set_cost(0.)

            else:
                outConn.set_cost(defined_steam_cost)

    def find_product_connections(self):

        try:

            for conn in self.base_block.output_connections:

                self.__check_connection(conn)

            self.__set_comp_cost()

        except:

            a = 10

    def contains(self, element):

        if "Connection" in str(type(element)) or issubclass(type(element), Connection):

            return element in self.contained_connection

        else:

            return element in self.contained_blocks

    def calculate_coefficients(self, total_destruction):

        super(ProductBlock, self).calculate_coefficients(total_destruction)

        self.base_block.coefficients = self.coefficients
        self.base_block.exergy_analysis = self.exergy_analysis

    def __check_connection(self, conn):

        if conn.is_system_output:

            self.main_class.generate_product_connection(conn, from_product_block=self)

        else:

            new_block = conn.to_block

            if not self.main_class.contains(new_block):

                if new_block.can_be_removed_in_pf_definition:

                    self.contained_connection.append(conn)
                    self.contained_blocks.append(new_block)

                    for conn in new_block.output_connections:

                        self.__check_connection(conn)

                else:

                    self.main_class.generate_product_block(new_block, input_connection=conn, from_block=self)

            else:

                self.main_class.generate_product_connection(conn, from_product_block=self, to_product_block=self.main_class.find_element(new_block))

    def __set_comp_cost(self):

        self.comp_cost = 0

        for block in self.contained_blocks:
            self.comp_cost += block.comp_cost


class ProductConnection(Connection):

    def __init__(self, base_connection: Connection):

        super().__init__(base_connection.ID)

        self.base_connection = base_connection

        self.exergy_value = base_connection.exergy_value
        self.rel_cost = base_connection.rel_cost

        self.is_useful_effect = base_connection.is_useful_effect
        self.is_fluid_stream = base_connection.is_fluid_stream

    @property
    def rel_cost(self) -> float:

        return self.__rel_cost

    @rel_cost.setter
    def rel_cost(self, rel_cost_input):

        self.__rel_cost = rel_cost_input
        self.base_connection.rel_cost = rel_cost_input


class PFArrayHandler(ArrayHandler):

    def __init__(self, base_array_handler: ArrayHandler):

        super().__init__()
        self.base_array_handler = base_array_handler
        self.__generate_lists()

    def __generate_lists(self):

        for connection in self.base_array_handler.system_inputs:

            new_block = connection.to_block
            self.generate_product_block(new_block, input_connection=connection)

    def generate_product_connection(self, input_connection: Connection, from_product_block=None, to_product_block=None):

        new_conn = ProductConnection(input_connection)
        self.append_connection(new_conn, from_block=from_product_block, to_block=to_product_block)

    def generate_product_block(self, input_block: Block, input_connection=None, from_block = None):

        new_block = self.find_element(input_block)

        if new_block is None:

            new_block = self.__append_new_product_block(input_block, input_connection, from_block)
            new_block.find_product_connections()

        elif input_connection is not None:

            self.generate_product_connection(input_connection, to_product_block=new_block, from_product_block=from_block)

    def __append_new_product_block(self, input_block: Block, input_connection, from_block) -> ProductBlock:

        new_block = ProductBlock(self.n_block, self, input_block)
        self.append_block(new_block)

        if input_connection is not None:
            self.generate_product_connection(input_connection, to_product_block=new_block, from_product_block=from_block)

        return new_block

    def find_element(self, element):

        for prod_block in self.block_list:

            if prod_block.contains(element):
                return prod_block

        return None

    def contains(self, element):

        return self.find_element(element) is not None



from EEETools.Tools.EESCodeGenerator.EES_parser import EESCodeAnalyzer
from EEETools.Tools.Other.fernet_handler import FernetHandler
from EEETools.Tools.modules_handler import ModulesHandler
from EEETools.costants import get_html_string
import xml.etree.ElementTree as ETree
import os


class Code:

    def __init__(self, modify_widget=None):

        self.modify_widget = modify_widget
        self.code_analyzer = EESCodeAnalyzer()
        self.modules_handler = ModulesHandler()

        self.blocks = list()
        self.inputs = dict()
        self.options = list()

        self.plain_text = ""
        self.rich_text = ""
        self.parsed_code = None

        self.update()

    def update(self):

        if self.modify_widget is not None:

            text = self.modify_widget.input_script.toPlainText()

        else:

            text = self.plain_text

        self.blocks = list()

        try:
            __block_list = self.code_analyzer.get_code_parts(text)

        except:
            self.blocks.append(CodeBlock(input_text=text))

        else:

            for __block_element in __block_list:
                __name = __block_element["name"]
                __type = __block_element["type"]
                __text = __block_element["expression"]

                self.blocks.append(CodeBlock(input_name=__name, input_type=__type, input_text=__text))

        self.plain_text = text
        self.set_rich_text()
        self.__update_inputs()

    def set_rich_text(self):

        self.rich_text = "<!DOCTYPE html>"
        self.rich_text += "<html><head>"
        self.rich_text += "<style>tab{margin-left: 10px;display:block;}</style>"
        self.rich_text += "</head><body><div contenteditable><p>"

        for line in self.blocks:
            self.rich_text += line.rich_text

        self.rich_text += "</p></div></body></html>"

    def __update_inputs(self):

        self.inputs = dict()

        for block in self.blocks:

            if block.has_input:
                self.inputs.update(block.inputs)

    def __update_options(self):

        self.options = list()

        for block in self.blocks:

            if block.type == "optional":
                self.options.append(block.name)

    # <----------------- READ & SAVE METHODS --------------->

    def save(self):

        # This class save the code data to a .dat file containing an encrypted xml tree. Encryption is useful in
        # order to prevent the user from modifying the file manually without going through the editor.
        #
        # The method generate the xml tree and pass it to the "__save_to_file" method that is in charge of the
        # encryption and file saving processes.
        #
        # here is an example of EES code xml file:
        #
        #   <data>
        #
        #     <header name="standard" component_type="Expander">
        #
        #         <options>
        #
        #             <option name="set_delta_P"/>
        #
        #         </options>
        #         <inputs>
        #
        #             <input name="Eff",     index="1",  related_option = "none"/>
        #             <input name="Delta_P", index="0",  related_option = "set_delta_P"/>
        #
        #         </inputs>
        #
        #         <variables>
        #
        #             <variable name="power input",  index="0",  multiple_check="False" />
        #             <variable name="flow input",   index="1",  multiple_check="False" />
        #             <variable name="flow output",  index="2",  multiple_check="False" />
        #
        #         </variables>
        #
        #     </header>
        #
        #     <EESCode>
        #
        #       "Turbine "
        #       Eff[$block_index] = $input[0]
        #
        #       &optional(set_delta_P){
        #
        #           Delta_P[$block_index] = $input[1]
        #           P[$2] = P[$1] - Delta_P[$block_index]
        #
        #       }
        #
        #       h_iso[$2] = enthalpy($fluid, P = P[$1], s = s[$1])
        #
        #       h[$2] = h[$1] + (h_iso[$2] - h[$1]) * Eff[$block_index]
        #       s[$2] = entropy($fluid, P = P[$2], h = h[$2])
        #       T[$2] = temperature($fluid, P = P[$2], h = h[$2])
        #
        #       m_dot[$2] = m_dot[$1]
        #       W[$0] = (h[$1] - h[$2]) * m_dot[$1]
        #
        #     </EESCode>
        #
        #   </data>

        if self.modify_widget is not None:

            data = ETree.Element("data")

            # <--------- HEADER DEFINITION --------->
            header = ETree.SubElement(data, "header")

            code_name = self.modify_widget.name_edit_text.text().lower()
            component_type = self.modify_widget.type_combobox.currentText()

            header.set("name", code_name)
            header.set("component_type", component_type)

            # <--------- OPTIONS DEFINITION --------->
            options = ETree.SubElement(header, "options")

            for option in self.options:
                input_element = ETree.SubElement(options, "option")
                input_element.set("name", str(option))

            # <--------- INPUTS DEFINITION --------->
            inputs = ETree.SubElement(header, "inputs")

            inputs_dict = self.inputs
            for key in inputs_dict.keys():
                input_element = ETree.SubElement(inputs, "input")
                input_element.set("name", str(key))
                input_element.set("index", str(inputs_dict[key][0]))
                input_element.set("related_option", str(inputs_dict[key][1]))

            # <--------- VARIABLES DEFINITION --------->
            variables = ETree.SubElement(header, "variables")

            variable_dict = self.modify_widget.tables["Input Indices"].model().data_dict
            for key in variable_dict.keys():
                variable = ETree.SubElement(variables, "variable")
                variable.set("name", str(key))
                variable.set("index", str((variable_dict[key])[0]))
                variable.set("multiple_check", str((variable_dict[key])[1]))

            # <--------- EES CODE DEFINITION --------->
            EES_code = ETree.SubElement(data, "EESCode")
            EES_code.text = self.plain_text

            # <--------- FILE PATH DEFINITION --------->
            main_folder = self.modules_handler.data_folder
            folder_name = self.modules_handler.get_module_name(component_type)
            file_name = code_name + ".dat"

            file_path = os.path.join(main_folder, folder_name, file_name)

            self.__save_to_file(data, file_path)

        else:

            folder_name = ""
            file_name = ""

        return [folder_name, file_name]

    def read_file(self, file_path):

        root = self.__read_file(file_path)

        EES_code = root.find("EESCode").text
        self.plain_text = EES_code

        return root

    @classmethod
    def rename_file(cls, new_name, file_path):

        root = cls.__read_file(file_path)

        header = root.find("header")
        header.attrib["name"] = new_name

        cls.__save_to_file(root, file_path)

    @staticmethod
    def __read_file(file_path):

        if ".xml" not in file_path:

            fernet = FernetHandler()
            root = fernet.read_file(file_path)

        else:

            data = file_path
            tree = ETree.parse(data)
            root = tree.getroot()

        return root

    @staticmethod
    def __save_to_file(root: ETree.Element, file_path):

        fernet = FernetHandler()
        fernet.save_file(file_path, root)


class CodeBlock:

    def __init__(self, input_type="default", input_name="", input_text=""):

        self.code_analyzer = EESCodeAnalyzer()
        self.modules_handler = ModulesHandler()

        self.lines = list()
        self.inputs = dict()

        self.type = input_type
        self.name = input_name

        self.plain_text = input_text
        self.rich_text = ""
        self.parsed_code = None

        self.update(self.plain_text)

    def update(self, text):

        self.lines = list()
        self.plain_text = text

        if not text == "":

            for line_text in text.splitlines():
                new_code_line = CodeLine(self.code_analyzer, line_text)
                self.lines.append(new_code_line)

            self.set_rich_text()
            self.__update_inputs()

    def set_rich_text(self):

        self.rich_text = ""
        __tabulation = ""

        if self.type == "optional":
            self.rich_text += "<br>"
            self.rich_text += get_html_string("repeated_keyword", "&optional")
            self.rich_text += get_html_string("default", "(")
            self.rich_text += get_html_string("variable", self.name)
            self.rich_text += get_html_string("default", ")")
            self.rich_text += get_html_string("default", "{")
            self.rich_text += "<tab>"

        self.rich_text += self.lines[0].rich_text

        if len(self.lines) > 1:

            for line in self.lines[1:]:
                self.rich_text += "<br>"
                self.rich_text += line.rich_text

        if self.type == "optional":
            self.rich_text += "</tab>"
            self.rich_text += "<br>"
            self.rich_text += get_html_string("default", "}")

    def __update_inputs(self):

        self.inputs = dict()

        if self.type == "default":
            __related_option = "none"

        else:
            __related_option = self.name

        for line in self.lines:

            if line.has_input:
                self.inputs.update(line.inputs)

        for key in self.inputs.keys():
            self.inputs[key].append(__related_option)

    @property
    def has_input(self):

        return not len(self.inputs) == 0


class CodeLine:

    def __init__(self, code_analyzer: EESCodeAnalyzer, plain_text: str):

        self.code_analyzer = code_analyzer
        self.parsed_text = None
        self.plain_text = plain_text
        self.rich_text = ""
        self.comment = ""

        self.inputs = dict()
        self.variables = list()

        self.__parse_text()

    def __parse_text(self):

        try:

            self.parsed_text = self.code_analyzer.parse_string(self.plain_text)

        except:

            self.has_error = True
            self.parsed_text = None
            self.rich_text = get_html_string("error", self.plain_text)

        else:

            self.has_error = False

            if not self.parsed_text is None:
                self.rich_text = self.parsed_text.rich_text

        self.__find_variables()

    def __find_variables(self):

        self.inputs = dict()
        self.variables = list()

        if self.parsed_text is not None:

            __tmp_variables = list()
            self.parsed_text.append_variable_to_list(__tmp_variables)

            for variable in __tmp_variables:

                self.variables.append(variable)

                if variable.is_input:
                    self.inputs.update({variable.name: [variable.index]})

    @property
    def has_input(self):

        return not len(self.inputs) == 0


from EEETools.Tools.CostCorrelations.cost_correlation_classes import AbstractCostCorrelation
from EEETools.Tools.CostCorrelations.cost_correlation_classes import Parameter
from EEETools.Tools.CostCorrelations.cost_correlation_gui import BaseEditCorrelationWidget
from EEETools.Tools.CostCorrelations.cost_correlation_gui import CoefficientWidget
from EEETools.Tools.CostCorrelations.cost_correlation_gui import ParametersWidget
from PyQt5.QtGui import QFont, QDoubleValidator
import xml.etree.ElementTree as ETree
from PyQt5.QtWidgets import *
import math


class TurtonCorrelation(AbstractCostCorrelation):

    def __init__(self, correlation_data=None):

        super().__init__()

        self.name = ""
        self.note = ""
        self.material = "Carbon Steel"
        self.component_type = ""

        self.__A = Parameter("A")
        self.__P = Parameter("P")

        self.__K = list()
        self.__C = list()
        self.__B = list()

        self.__F_m = 0.
        self.__current_cepci = 0
        self.__correlation_cepci = None

        self.is_imported_from_file = False

        if correlation_data is not None:
            self.import_xml_tree(correlation_data)

    def get_cost(self):

        # This correlation format is extracted from: Turton, R, Bailie, R C, & Whiting, W B. "Analysis, synthesis and
        # design of chemical processes" Appendix A - pg.924.
        #
        # Once calculated, the cost value value is actualized using the CEPCI value relation

        CP_0 = self.__K[0]
        CP_0 += self.__K[1] * math.log10(self.__A.value)
        CP_0 += self.__K[2] * math.pow(math.log10(self.__A.value), 2)

        CP_0 = math.pow(10, CP_0)
        F_p = self.__get_F_p_coefficient()
        F_m = self.__F_m

        C_BM = CP_0 * (self.__B[0] + self.__B[1] * F_p * F_m)

        return C_BM * self.__current_cepci / self.__correlation_cepci

    def __get_F_p_coefficient(self):

        # If the material IS NOT carbon steel other parameters are required:
        #
        #   - C[0], C[1] and C[2] are interpolation parameters and could be retrieved in Turton book
        #   - __P.value is pressure value as defined in the correlation description
        #   - F_p is the pressure factor defined according to the equipment type

        F_p = self.__C[0]
        F_p += self.__C[1] * math.log10(self.__P.value)
        F_p += self.__C[2] * math.pow(math.log10(self.__P.value), 2)

        return math.pow(10, F_p)

    @property
    def parameter_dict(self):

        param_dict = {"A": self.__A,
                      "P": self.__P}

        return param_dict

    def set_parameters(self, input_dict: dict):

        if "A" in input_dict.keys():
            self.__A.value = input_dict["A"]

        if "P" in input_dict.keys():
            self.__P.value = input_dict["P"]

    @property
    def coefficient_dict(self):

        coefficients = {"corr_CEPCI": {"array size": 1, "is optional": False},
                        "F_m": {"array size": 1, "is optional": False},
                        "K": {"array size": 3, "is optional": False},
                        "C": {"array size": 3, "is optional": False},
                        "B": {"array size": 2, "is optional": True}}

        return coefficients

    @property
    def coefficients(self):

        return {"curr_CEPCI": [self.__correlation_cepci],
                "F_m": [self.__F_m],
                "K": self.__K,
                "C": self.__C,
                "B": self.__B}

    @coefficients.setter
    def coefficients(self, input_dict: dict):

        if "corr_CEPCI" in input_dict.keys():
            self.__correlation_cepci = input_dict["corr_CEPCI"][0]

        if "F_m" in input_dict.keys():
            self.__F_m = input_dict["F_m"][0]

        if "K" in input_dict.keys():
            self.__K = input_dict["K"]

        if "C" in input_dict.keys():
            self.__C = input_dict["C"]

        if "B" in input_dict.keys():
            self.__B = input_dict["B"]

    @property
    def description(self):
        return ""

    @property
    def edit_correlation_widget(self) -> QWidget:
        return TurtonEditCorrelationWidget(self)

    @property
    def parameter_settings_widget(self):
        return None

    def import_xml_tree(self, data: ETree.Element):

        header = data.find("header")

        self.name = header.get("name")
        self.component_type = header.get("component_type")
        self.note = header.get("note")

        parametersList = data.findall("parameter")

        for parameter in parametersList:

            __tmp_parameter = Parameter()
            __tmp_parameter.measure_unit = parameter.get("measure_unit")
            __tmp_parameter.range = parameter.get("range")
            __tmp_parameter.note = parameter.get("note")

            param_dict = {parameter.get("name"): __tmp_parameter}
            self.set_parameters(param_dict)

        coefficientsList = data.findall("coefficient")

        for coefficient in coefficientsList:

            __tmp_list = list()

            for element in coefficient.findall("element"):

                __tmp_list.append(float(element.get("value")))

            coeff_dict = {coefficient.get("name"): __tmp_list}
            self.coefficients = coeff_dict

    @property
    def export_xml_tree(self):

        # This class save the correlation data to a .dat file containing an encrypted xml tree. Encryption is useful in
        # order to prevent the user from modifying the file manually without going through the editor.
        #
        # The method generate the xml tree and pass it to the "export_to_file" method that is in charge of the actual
        # encryption and file saving processes.
        #
        # here is an example of turton correlation xml file:
        #
        #   <data>
        #
        #     <header
        #
        #         correlation_class = "Turton Correlation"
        #         name = "standard"
        #         component_type = "Heat Exchanger"
        #         note = "Plate Heat Exchanger"
        #
        #         ></header>
        #
        #     <parameters>
        #
        #          <parameter
        #
        #              name = "A"
        #              measure_unit = "m^2"
        #              range = "30-60"
        #              note = "Heat Exchanger Area"
        #
        #              ></parameter>
        #
        #          <parameter
        #
        #              name = "P"
        #              measure_unit = "kPa"
        #              range = "300-1000"
        #              note = "Mean Pressure"
        #
        #              ></parameter>
        #
        #     </parameters>
        #
        #     <coefficients>
        #
        #       <coefficient name = "corr_CEPCI">
        #
        #           <element value = 357>
        #
        #       </coefficient>
        #
        #       <coefficient name = "K">
        #
        #           <element value = 1.2039>
        #           <element value = 5.6434>
        #           <element value = 0.0353>
        #
        #       </coefficient>
        #
        #       <coefficient name = "C">
        #
        #           <element value = 3.4510">
        #           <element value = 0.7601">
        #           <element value = 0.0032">
        #
        #       </coefficient>
        #
        #       <coefficient name = "B">
        #
        #           <element value = 0.0012">
        #           <element value = 3.4550">
        #
        #       </coefficient>
        #
        #     </coefficients>
        #
        #   </data>

        data = ETree.Element("data")

        # <--------- HEADER DEFINITION --------->
        header = ETree.SubElement(data, "header")

        correlation_name = self.name
        correlation_note = self.note
        component_type = self.component_type

        header.set("correlation_class", "Turton Correlation")
        header.set("name", correlation_name)
        header.set("component_type", component_type)
        header.set("note", correlation_note)

        # <--------- PARAMETERS DEFINITION --------->
        parameters = ETree.SubElement(data, "parameters")

        for key in self.parameter_dict.keys():

            parameter = ETree.SubElement(parameters, "parameter")

            parameter.set("name", key)
            parameter.set("measure_unit", self.parameter_dict[key].measure_unit)
            parameter.set("range", str(self.parameter_dict[key].range))
            parameter.set("note", str(self.parameter_dict[key].note))

        # <--------- COEFFICIENTS DEFINITION --------->
        coefficients = ETree.SubElement(data, "coefficients")

        for key in self.coefficients.keys():

            coefficient = ETree.SubElement(coefficients, "coefficient")
            coefficient.set("name", key)

            for element_value in self.coefficients[key]:
                element = ETree.SubElement(coefficient, "element")
                element.set("value", str(element_value))

        return data


class TurtonEditCorrelationWidget(BaseEditCorrelationWidget):

    def __init__(self, cost_correlation_class: AbstractCostCorrelation):

        super().__init__(cost_correlation_class)
        self.title = "Turton Cost Correlation"

        self.header_widget = None
        self.parameter_widget_dict = dict()

        self.__set_header_layout()
        self.__set_parameter_layout()

    def __update(self):

        coeff_dict = dict()

        for key in self.coefficient_widget_dict.keys():

            coeff_dict.update(self.coefficient_widget_dict[key].coefficient_dict)

        self.cost_correlation.coefficients = coeff_dict

    def __reset_dialog(self):
        self.header_widget.component_type_combo.setEnabled(True)

    @property
    def is_ready_for_saving(self):

        for sel_dict in [self.parameter_widget_dict]:

            for key in sel_dict.keys():

                if not sel_dict[key].is_ready_for_saving:

                    return False

        return self.header_widget.is_ready_for_saving

    def __set_header_layout(self):

        widget = TurtonHeaderWidget(self)
        widget.set_edit_layout = True
        widget.init_layout()

        self.header_widget = widget

    def __set_parameter_layout(self):

        parameter_dict = self.cost_correlation.parameter_dict
        parameter_layout = QHBoxLayout()

        for key in parameter_dict.keys():

            widget = ParametersWidget(parameter_dict[key], self)
            widget.set_edit_layout = True
            widget.init_layout()

            self.parameter_widget_dict.update({key: widget})
            parameter_layout.addWidget(widget)

        parameter_widget = QWidget()
        parameter_widget.setLayout(parameter_layout)

        self.parameter_widget = parameter_widget


class TurtonHeaderWidget(QWidget):

    def __init__(self, parent_class: TurtonEditCorrelationWidget):

        super().__init__(None)

        self.correlation = parent_class.cost_correlation
        self.set_edit_layout = False

        self.parent_class = parent_class

        self.main_layout = QVBoxLayout()
        self.grid_layout = QGridLayout()

    def init_layout(self):

        if self.set_edit_layout:
            self.__init_edit_objects()
            self.__init_edit_layout()

        else:
            self.__init_set_objects()
            self.__init_set_layout()

        self.setLayout(self.main_layout)

    def __init_edit_objects(self):

        self.name_edit = QLineEdit()
        self.note_edit = QTextEdit()
        self.component_type_combo = QComboBox()

        self.name_edit.setText(self.correlation.name)
        self.note_edit.setText(self.correlation.note)

        from EEETools.Tools.modules_handler import ModulesHandler
        modules_handler = ModulesHandler()

        self.component_type_combo.addItems(modules_handler.name_list)
        self.component_type_combo.setEnabled(False)

        if not self.correlation.component_type == "":

            component_type = self.correlation.component_type
            self.component_type_combo.setCurrentIndex(modules_handler.get_name_index(component_type))

        else:

            self.component_type_combo.setCurrentIndex(0)

        self.name_edit.setObjectName("name_edit")
        self.note_edit.setObjectName("note_edit")
        self.component_type_combo.setObjectName("component_type_combo")

        self.name_edit.textChanged.connect(self.on_text_edit_change)
        self.note_edit.textChanged.connect(self.on_text_edit_change)
        self.component_type_combo.currentTextChanged.connect(self.on_text_edit_change)

    def __init_set_objects(self):

        self.curr_cepci_edit = QLineEdit()
        self.name_edit = QLabel()
        self.note_edit = QLabel()
        self.component_type_combo = QLabel()

        self.note_edit.setWordWrap(True)
        self.curr_cepci_edit.setValidator(QDoubleValidator())
        self.curr_cepci_edit.setObjectName("curr_cepci_edit")
        self.curr_cepci_edit.textChanged.connect(self.on_text_edit_change)

        self.note_edit.setText(self.correlation.name)
        self.note_edit.setText(self.correlation.note)
        self.note_edit.setText(self.correlation.component_type)

    def __init_edit_layout(self):

        # <----------- FIRST ROW ----------->

        name_label = QLabel()
        name_label.setText("Correlation Name:")

        self.grid_layout.addWidget(name_label, 0, 0)
        self.grid_layout.addWidget(self.name_edit, 0, 1)

        # <---------- SECOND ROW ----------->

        component_type_label = QLabel()
        component_type_label.setText("Component Type:")

        self.grid_layout.addWidget(component_type_label, 1, 0)
        self.grid_layout.addWidget(self.component_type_combo, 1, 1)

        # <----------- FINAL ROW ----------->

        grid_layout_widget = QWidget()
        grid_layout_widget.setLayout(self.grid_layout)

        self.main_layout.addWidget(grid_layout_widget)
        self.main_layout.addWidget(self.note_edit)

    def __init_set_layout(self):

        # <----------- FIRST ROW ----------->

        name_label = QLabel()
        name_label.setText("Correlation Name:")

        self.grid_layout.addWidget(name_label, 0, 0)
        self.grid_layout.addWidget(self.name_edit, 0, 1)

        # <---------- SECOND ROW ----------->

        component_type_label = QLabel()
        component_type_label.setText("Component Type:")

        self.grid_layout.addWidget(component_type_label, 1, 0)
        self.grid_layout.addWidget(self.component_type_combo, 1, 1)

        # <---------- THIRD ROW ----------->

        curr_cepci_label = QLabel()
        curr_cepci_label.setText("Current CEPCI:")

        self.grid_layout.addWidget(curr_cepci_label, 2, 0)
        self.grid_layout.addWidget(self.curr_cepci_edit, 2, 1)

        # <----------- FINAL ROW ----------->

        grid_layout_widget = QWidget()
        grid_layout_widget.setLayout(self.grid_layout)

        self.main_layout.addWidget(grid_layout_widget)
        self.main_layout.addWidget(self.note_edit)

    def on_text_edit_change(self):

        name = self.sender().objectName()

        if name == "curr_cepci_edit":

            self.correlation.currentCEPCI = float(self.curr_cepci_edit.text())

        elif name == "name_edit":

            self.correlation.name = self.note_edit.text()

        elif name == "note_edit":

            self.correlation.note = self.note_edit.toPlainText()

        elif name == "component_type_combo":

            self.correlation.component_type = self.component_type_combo.currentText()

        self.parent_class.enable_buttons()

    @property
    def is_ready_for_saving(self):

        for edit_text in [self.name_edit, self.note_edit]:

            if edit_text.text() == "":
                return False

        return True


import win32com.client


class UNISIMConnector:

    def __init__(self, filepath=None):

        self.app = win32com.client.Dispatch('UnisimDesign.Application')

        self.__doc = None
        self.solver = None
        self.block_names = list()
        self.stream_names = list()

        self.open(filepath)

    def __enter__(self):

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):

        try:

            self.doc.Visible = False
            self.__doc.Close()

        except:

            pass

        self.app.Visible = False
        self.app.Quit()

    def get_spreadsheet(self, spreadsheet_name):

        try:

            spreadsheet = UNISIMSpreadsheet(self, spreadsheet_name)

        except:

            return None

        return spreadsheet

    def generate_excel_file(self):

        pass

    def show(self):

        try:

            self.app.Visible = True
            self.doc.Visible = True

        except:

            pass

    def open(self, filepath):

        if self.is_ready:

            self.__doc.Close()

        if filepath is None:

            self.doc = self.app.SimulationCases.Add()

        else:

            self.doc = self.app.SimulationCases.Open(filepath)

    def get_block(self, block_name):

        if block_name not in self.block_names:

            return None

        else:

            return self.doc.Flowsheet.Operations.Item(block_name)

    def get_stream(self, stream_name):

        if stream_name not in self.stream_names:

            return None

        else:

            return self.doc.Flowsheet.Streams.Item(stream_name)

    @property
    def doc(self):

        return self.__doc

    @doc.setter
    def doc(self, doc):

        try:

            self.solver = doc.Solver

        except:

            self.__doc = None
            self.solver = None
            self.block_names = list()
            self.stream_names = list()

        else:

            self.__doc = doc
            self.block_names = self.doc.Flowsheet.Operations.Names
            self.stream_names = self.doc.Flowsheet.Streams.Names

    @property
    def is_ready(self):

        return self.__doc is not None

    def wait_solution(self):

        try:

            while self.solver.IsSolving:

                pass

        except:

            pass


class UNISIMSpreadsheet:

    def __init__(self, parent:UNISIMConnector, spreadsheet_name):

        self.parent = parent
        self.spreadsheet = parent.doc.Flowsheet.Operations.Item(spreadsheet_name)

    def set_cell_from_list(self, input_list: list):

        for value_dict in input_list:

            self.set_cell_value(value_dict["cell name"], value_dict["value"])

    def get_value_from_list(self, input_list: list):

        return_dict = dict()

        for cell_name in input_list:

            return_dict.update({cell_name: self.get_cell_value(cell_name)})

    def set_cell_value(self, cell_name, value):

        try:

            cell = self.spreadsheet.Cell(cell_name)
            cell.CellValue = value

        except:

            pass

    def get_cell_value(self, cell_name):

        try:

            cell = self.spreadsheet.Cell(cell_name)
            value = cell.CellValue

        except:

            return None

        return value


from EEETools.Tools.Other.unisim_connect import UNISIMConnector
from tkinter import filedialog
import tkinter as tk
import unittest


class MyTestCase(unittest.TestCase):

    def test_open_file(self):

        root = tk.Tk()
        root.withdraw()
        unisim_path = filedialog.askopenfilename()

        with UNISIMConnector(unisim_path) as unisim:

            a = unisim.get_block("COND")
            b = unisim.get_stream("4")
            print(a)


import importlib, inspect
from EEETools import costants


class Handler:

    """This is the base class for classes that have to handle a group of subclasses collected in a specific folder:

            - Import_correct_sub_class method return the correct subclass element from a given name
            - Other methods are related with handling of the format for module and class names

        !!!ATTENTION!!! In creating the sub-classes the user must care to overload:

            - "self.data_folder" variable with the path to the folder containing the subclasses (e.g.
            "EEETools\BlockSubClasses")

            - "self.subclass_directory_path" variable variable with the path to the folder containing the subclasses
            using dots as separators ( e.g. "EEETools.BlockSubClasses") """

    def __init__(self):

        self.current_folder = costants.ROOT_DIR
        self.data_folder = self.current_folder

        self.__subclass_directory_path = ""
        self.__subclass_folder_path = ""

    @property
    def subclass_directory_path(self):

        return self.__subclass_directory_path

    @subclass_directory_path.setter
    def subclass_directory_path(self, input_path):

        self.__subclass_directory_path = input_path
        self.__subclass_folder_path = input_path + "."

    def import_correct_sub_class(self, subclass_name):

        """This method import the correct block subclass.

        Subclasses modules must be placed inside "EEETools.BlockSubClasses" Package. Subclasses
        name must be capitalized. In addition, subclass module name must be the same but without capital letters e.g
        subClasses "Expander" must be in module "expander". The static function called "__get_subclass_module_names"
        generates from "subclass_name" the correct "__module_name" and "__subclass_name" """

        __tmp_result = self.__get_subclass_module_names(subclass_name)
        __module_name = __tmp_result[0]
        __subclass_name = __tmp_result[1]

        __importString = self.__subclass_folder_path + __module_name
        __blockModule = importlib.import_module(__importString)
        return getattr(__blockModule, __subclass_name)

    def get_module_name(self, std_name: str):

        return self.__get_subclass_module_names(std_name)[0]

    def check_data_folder(self):
        pass

    def list_modules(self) -> list:

        raw_names = self.__get_raw_names()

        for i in range(len(raw_names)):

            name = raw_names[i]

            if "_" in name:

                __tmp_list = name.split("_")
                new_name = __tmp_list[0].capitalize()

                for __tmp_array in __tmp_list[1:]:
                    new_name += " " + __tmp_array.capitalize()

            else:

                new_name = name.capitalize()

            raw_names[i] = new_name

        return raw_names

    def __get_raw_names(self) -> list:

        member_list = list()
        __members = inspect.getmembers(importlib.import_module(self.__subclass_directory_path))

        for __member in __members:

            if "__" not in __member[0] and __member[0].islower():
                member_list.append(__member[0])

        return member_list

    @staticmethod
    def get_std_name(input_name: str):

        # This method performs the reverse operation with respect to "__get_subclass_module_names" module.
        # "input_name" could have two format:
        #
        # "__module_name" format:
        #
        #       - in case of multi-word names (e.g. "Heat Exchanger") the words is connected by an underscore ("_")
        #       - each word must be lowercase
        #       - for example: "Heat Exchanger" -> "heat_exchanger"
        #
        # "__subclass_name" format:
        #
        #       - in case of multi-word names (e.g. "Heat Exchanger") the space is removed
        #       - each word must be capitalized
        #       - for example: "Heat Exchanger" -> "HeatExchanger"
        #
        # The method returns the name in the standard format:
        #
        #       - in case of multi-word names (e.g. "Heat Exchanger") the words are separeted by a space (" ")
        #       - each word must be capitalized
        #       - for example: "heat_exchanger" -> "Heat Exchanger"
        #
        # Whatever the input format the program scrolls the string searching for "_" or for uppercase letters:
        #
        #       - if "_" is found, it is replaced with a space " "
        #       - if an uppercase letter is found, a space " " is inserted in the string before the letter.
        #       - otherwise the letter is simply copied into "__std_name" string

        __name_index = list(input_name)
        __std_name = str(__name_index[0]).capitalize()
        capitalize_next = False

        for letter in __name_index[1:]:

            if letter == "_":

                __std_name += " "
                capitalize_next = True

            elif letter.isupper():

                __std_name += " " + str(letter)

            else:

                if capitalize_next:
                    __std_name += str(letter).capitalize()
                    capitalize_next = False

                else:
                    __std_name += str(letter)

        return __std_name

    @staticmethod
    def __get_subclass_module_names(input_name: str):

        # this method generates from "input_name" the correct "__module_name" and "__subclass_name".
        #
        # "__module_name" must have the following format:
        #
        #       - in case of multi-word names (e.g. "Heat Exchanger") the words must be connected by an underscore ("_")
        #       - each word must be lowercase
        #       - for example: "Heat Exchanger" -> "heat_exchanger"
        #
        # "__subclass_name" must have the following format:
        #
        #       - in case of multi-word names (e.g. "Heat Exchanger") the space must be removed
        #       - each word must be capitalized
        #       - for example: "Heat Exchanger" -> "HeatExchanger"

        __tmp_list = input_name.split(" ")

        if len(__tmp_list) == 1:

            __module_name = input_name.lower()
            __subclass_name = input_name.lower().capitalize()

        else:

            __module_name = __tmp_list[0].lower()
            __subclass_name = __tmp_list[0].lower().capitalize()

            for string in __tmp_list[1:]:
                __module_name += "_" + string.lower()
                __subclass_name += string.lower().capitalize()

        return [__module_name.strip(), __subclass_name]


from EEETools.MainModules import ArrayHandler


def get_result_data_frames(array_handler: ArrayHandler) -> dict:

    # Stream Solution Data frame generation
    stream_data = {"Stream": list(),
                   "Name": list(),
                   "Exergy Value [kW]": list(),
                   "Specific Cost [€/kJ]": list(),
                   "Specific Cost [€/kWh]": list(),
                   "Total Cost [€/s]": list()}

    for conn in array_handler.connection_list:

        if not conn.is_internal_stream:
            stream_data["Stream"].append(conn.index)
            stream_data["Name"].append(conn.name)
            stream_data["Exergy Value [kW]"].append(conn.exergy_value)
            stream_data["Specific Cost [€/kJ]"].append(conn.rel_cost)
            stream_data["Specific Cost [€/kWh]"].append(conn.rel_cost * 3600)
            stream_data["Total Cost [€/s]"].append(conn.rel_cost * conn.exergy_value)

    # Components Data frame generation
    comp_data = {"Name": list(),
                 "Comp Cost [€/s]": list(),

                 "Exergy_fuel [kW]": list(),
                 "Exergy_product [kW]": list(),
                 "Exergy_destruction [kW]": list(),
                 "Exergy_loss [kW]": list(),
                 "Exergy_dl [kW]": list(),

                 "Fuel Cost [€/kWh]": list(),
                 "Fuel Cost [€/s]": list(),
                 "Product Cost [€/kWh]": list(),
                 "Product Cost [€/s]": list(),
                 "Destruction Cost [€/kWh]": list(),
                 "Destruction Cost [€/s]": list(),

                 "eta": list(),
                 "r": list(),
                 "f": list(),
                 "y": list()}

    for block in array_handler.block_list:

        if not block.is_support_block:
            comp_data["Name"].append(block.name)
            comp_data["Comp Cost [€/s]"].append(block.comp_cost)

            comp_data["Exergy_fuel [kW]"].append(block.exergy_analysis["fuel"])
            comp_data["Exergy_product [kW]"].append(block.exergy_analysis["product"])
            comp_data["Exergy_destruction [kW]"].append(block.exergy_analysis["distruction"])
            comp_data["Exergy_loss [kW]"].append(block.exergy_analysis["losses"])
            comp_data["Exergy_dl [kW]"].append(block.exergy_analysis["distruction"] + block.exergy_analysis["losses"])

            try:

                comp_data["Fuel Cost [€/kWh]"].append(block.coefficients["c_fuel"] * 3600)
                comp_data["Product Cost [€/kWh]"].append(block.output_cost * 3600)
                comp_data["Destruction Cost [€/kWh]"].append(block.coefficients["c_dest"] * 3600)

                comp_data["Fuel Cost [€/s]"].append(block.coefficients["c_fuel"] * block.exergy_analysis["fuel"])
                comp_data["Product Cost [€/s]"].append(block.output_cost * block.exergy_analysis["fuel"])
                comp_data["Destruction Cost [€/s]"].append(
                    block.coefficients["c_dest"] * (block.exergy_analysis["distruction"] + block.exergy_analysis["losses"]))

                comp_data["eta"].append(block.coefficients["eta"])
                comp_data["r"].append(block.coefficients["r"])
                comp_data["f"].append(block.coefficients["f"])
                comp_data["y"].append(block.coefficients["y"])

            except:

                comp_data["Fuel Cost [€/kWh]"].append(0)
                comp_data["Product Cost [€/kWh]"].append(0)
                comp_data["Destruction Cost [€/kWh]"].append(0)

                comp_data["Fuel Cost [€/s]"].append(0)
                comp_data["Product Cost [€/s]"].append(0)
                comp_data["Destruction Cost [€/s]"].append(0)

                comp_data["eta"].append(0)
                comp_data["r"].append(0)
                comp_data["f"].append(0)
                comp_data["y"].append(0)

    # Output Stream Data frame generation
    useful_data = {"Stream": list(),
                   "Name": list(),
                   "Exergy Value [kW]": list(),
                   "Specific Cost [€/kJ]": list(),
                   "Specific Cost [€/kWh]": list(),
                   "Total Cost [€/s]": list()}

    for conn in array_handler.useful_effect_connections:

        useful_data["Stream"].append(conn.index)
        useful_data["Name"].append(conn.name)
        useful_data["Exergy Value [kW]"].append(conn.exergy_value)
        useful_data["Specific Cost [€/kJ]"].append(conn.rel_cost)
        useful_data["Specific Cost [€/kWh]"].append(conn.rel_cost * 3600)
        useful_data["Total Cost [€/s]"].append(conn.rel_cost * conn.exergy_value)

    return {"Stream Out": stream_data,
            "Comp Out": comp_data,
            "Eff Out": useful_data}


import os
import unittest
from EEETools.Tools.modules_handler import ModulesHandler

class ModulesHandlerTestCase(unittest.TestCase):

    def testInitializeList(self):

        modules_handler = ModulesHandler()
        print(modules_handler.name_list)

        #If there were no errors the test succedeed
        self.assertTrue(True)

    def testConvertName(self):

        modules_handler = ModulesHandler()

        for filename in os.listdir(modules_handler.data_folder):

            index = modules_handler.get_name_index(filename)

            self.assertNotEqual(-1, index)

            std_name = modules_handler.name_list[index]
            module_name = modules_handler.get_module_name(std_name)

            self.assertEqual(filename, module_name)


if __name__ == '__main__':
    unittest.main()



from PyQt5.QtCore import QAbstractTableModel, QModelIndex, Qt
from PyQt5.QtGui import QColor, QContextMenuEvent, QIcon
from PyQt5.QtWidgets import *
import os


class AbstractFilesManager(QDialog):

    def __init__(self, flags=None, *args, **kwargs):

        super().__init__(flags, *args, **kwargs)

        self.file_edit = None
        self.title = ""
        self.modules_handler = None
        self.selected_element_path = ""

    def __init_file_tree(self, dir_path):

        layout = QVBoxLayout()

        layout.addWidget(self.__get_tree_widget(dir_path))
        layout.addWidget(self.__get_buttons_widget())

        self.setLayout(layout)

    def __get_tree_widget(self, dir_path) -> QWidget:

        self.model = QFileSystemModel()
        self.model.setRootPath(dir_path)

        self.tree = QTreeView()
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(dir_path))
        self.tree.clicked.connect(self.on_selection_changed)
        self.tree.setColumnWidth(0, 250)

        return self.tree

    def __get_buttons_widget(self) -> QWidget:

        buttons_widget = QWidget()
        v_layout = QHBoxLayout()

        self.add_button = QPushButton()
        self.add_button.setText("Add")
        self.add_button.setEnabled(False)
        self.add_button.clicked.connect(self.on_button_add_pressed)

        self.rename_button = QPushButton()
        self.rename_button.setText("Rename")
        self.rename_button.clicked.connect(self.on_button_rename_pressed)
        self.rename_button.setEnabled(False)

        self.modify_button = QPushButton()
        self.modify_button.setText("Modify")
        self.modify_button.clicked.connect(self.on_button_modify_pressed)
        self.modify_button.setEnabled(False)

        self.delete_button = QPushButton()
        self.delete_button.setText("Delete")
        self.delete_button.clicked.connect(self.on_button_delete_pressed)
        self.delete_button.setEnabled(False)

        self.duplicate_button = QPushButton()
        self.duplicate_button.setText("Duplicate")
        self.duplicate_button.clicked.connect(self.on_button_duplicate_pressed)
        self.duplicate_button.setEnabled(False)

        v_layout.addWidget(self.add_button)
        v_layout.addWidget(self.rename_button)
        v_layout.addWidget(self.modify_button)
        v_layout.addWidget(self.delete_button)
        v_layout.addWidget(self.duplicate_button)

        buttons_widget.setLayout(v_layout)

        return buttons_widget

    def __enable_buttons(self):

        if not os.path.exists(self.selected_element_path):
            self.add_button.setEnabled(False)
            self.delete_button.setEnabled(False)
            self.rename_button.setEnabled(False)
            self.modify_button.setEnabled(False)
            self.duplicate_button.setEnabled(False)

        if os.path.isdir(self.selected_element_path):

            self.add_button.setEnabled(True)
            self.delete_button.setEnabled(False)
            self.rename_button.setEnabled(False)
            self.modify_button.setEnabled(False)
            self.duplicate_button.setEnabled(False)

        elif os.path.isfile(self.selected_element_path):

            self.add_button.setEnabled(False)
            self.delete_button.setEnabled(True)
            self.rename_button.setEnabled(True)
            self.modify_button.setEnabled(True)
            self.duplicate_button.setEnabled(True)

        else:

            self.add_button.setEnabled(False)
            self.delete_button.setEnabled(False)
            self.rename_button.setEnabled(False)
            self.modify_button.setEnabled(False)
            self.duplicate_button.setEnabled(False)

    def __rename_file(self, new_name, file_path):

        dir_path = os.path.dirname(file_path)
        new_path = os.path.join(dir_path, new_name + ".dat")

        self.selected_element_path = new_path
        os.rename(file_path, new_path)

    def on_selection_changed(self, signal):

        self.selected_element_path = self.model.filePath(signal)
        self.__enable_buttons()

    def on_button_add_pressed(self):
        raise NotImplementedError

    def on_button_rename_pressed(self):
        raise NotImplementedError

    def on_button_modify_pressed(self):
        raise NotImplementedError

    def on_button_delete_pressed(self):
        raise NotImplementedError

    def on_button_duplicate_pressed(self):
        raise NotImplementedError

    def show(self):

        self.setWindowTitle(self.title)
        self.setGeometry(300, 300, 650, 300)

        self.modules_handler.check_data_folder()
        self.selected_element_path = self.modules_handler.data_folder
        self.__init_file_tree(self.modules_handler.data_folder)

        super(AbstractFilesManager, self).show()


class WarningDialog(QDialog):

    # noinspection PyArgumentList
    def __init__(self, flags=None, *args, **kwargs):

        super().__init__(flags, *args, **kwargs)

        self.setWindowTitle("Warning")

        self.show_only_one_button = False
        self.button_on_click = None
        self.layout_is_set = False

        self.label = QLabel()
        self.yes_button = WarningButton()
        self.no_button = WarningButton()

        self.__init_layout()

    def __init_layout(self):

        self.button_on_click = self.__default_button_on_click
        self.yes_button.set_state(is_yes=True)
        self.no_button.set_state(is_yes=False)

    def set_label_text(self, text: str):
        self.label.setText(text)

    def set_layout(self, input_layout: QLayout = None):

        v_layout = QVBoxLayout()
        v_layout.addWidget(self.label)

        if not input_layout is None:
            __tmp_widget = QWidget()
            __tmp_widget.setLayout(input_layout)
            v_layout.addWidget(__tmp_widget)

        button_layout = QHBoxLayout()
        button_layout.addWidget(self.yes_button)

        if not self.show_only_one_button:
            button_layout.addWidget(self.no_button)

        __tmp_widget = QWidget()
        __tmp_widget.setLayout(button_layout)
        v_layout.addWidget(__tmp_widget)

        self.setLayout(v_layout)
        self.layout_is_set = True

    def set_button_text(self, text: str, set_button_yes=True):

        if set_button_yes:

            self.yes_button.setText(text)

        else:

            self.no_button.setText(text)

    def set_buttons_on_click(self, function):

        self.button_on_click = function

    def show(self):

        if not self.layout_is_set:
            self.set_layout()

        self.yes_button.clicked.connect(self.button_on_click)
        self.no_button.clicked.connect(self.button_on_click)

        super(WarningDialog, self).show()

    def __default_button_on_click(self):

        self.close()


class WarningButton(QPushButton):

    def __init__(self, flags=None, *args, **kwargs):
        super().__init__(flags, *args, **kwargs)
        self.is_yes_button = False

    def set_state(self, is_yes):
        self.is_yes_button = is_yes


class AbstractTable(QTableView):

    def __init__(self, main_window, title):

        super().__init__()

        new_model = self.table_model(main_window, title)
        self.setModel(new_model)

        self.currentRow = 0
        self.__set_actions()

    def contextMenuEvent(self, event: QContextMenuEvent):

        menu = QMenu(self)

        # Populating the menu with actions
        menu.addAction(self.add_action)
        menu.addAction(self.delete_action)
        menu.addAction(self.modify_action)

        self.currentRow = self.rowAt(event.pos().y())
        self.__enable_actions()

        # Launching the menu
        menu.exec(event.globalPos())

    def __set_actions(self):

        self.modify_action = QAction(self)
        self.modify_action.setText("&Modify")
        self.modify_action.setIcon(QIcon(":icon-modify"))
        self.modify_action.triggered.connect(self.onModifyPressed)

        self.delete_action = QAction(self)
        self.delete_action.setText("&Delete")
        self.delete_action.setIcon(QIcon(":icon-delete"))
        self.delete_action.triggered.connect(self.onDeletePressed)

        self.add_action = QAction(self)
        self.add_action.setText("&Add")
        self.add_action.setIcon(QIcon(":icon-add"))
        self.add_action.triggered.connect(self.onAddPressed)

    def __enable_actions(self):

        model = self.model()

        self.modify_action.setEnabled(model.context_action_activation(self.currentRow, "modify"))
        self.delete_action.setEnabled(model.context_action_activation(self.currentRow, "delete"))
        self.add_action.setEnabled(model.context_action_activation(self.currentRow, "add"))

    def onAddPressed(self):
        self.model().onAddClicked(self.currentRow)

    def onDeletePressed(self):
        self.model().onDeleteClicked(self.currentRow)

    def onModifyPressed(self):
        self.model().onModifyClicked(self.currentRow)

    @property
    def table_model(self):

        return IndexSetterTableModel


class IndexSetterTableModel(QAbstractTableModel):

    def __init__(self, main_window, type):

        super().__init__()

        self.type = type
        self.main_window = main_window

        self.row_count = 0
        if self.type in ["Input Indices", "Parameters Indices"]:
            self.column_count = 3

        else:
            self.column_count = 2

        self.data_dict = dict()

        self.additional_info_list = list()
        self.names_list = list()
        self.index_list = list()

        self.change_data_type()

    def change_data_type(self):

        if self.type == "Input Indices":

            __current_type_name = str(self.main_window.type_combobox.currentText())
            __current_type_class = self.main_window.modules_handler.import_correct_sub_class(__current_type_name)
            data: dict = __current_type_class.return_EES_needed_index()

        elif self.type == "Parameters Indices":

            data = dict()

        else:

            __current_type_name = str(self.main_window.block_ID_combobox.currentText())
            __current_type_class = self.main_window.modules_handler.import_correct_sub_class(__current_type_name)
            data: dict = __current_type_class.return_EES_base_equations()

        self.load_data(data)

    def load_data(self, data):

        self.names_list = list()
        self.index_list = list()
        self.additional_info_list = list()

        self.data_dict = data

        for key in data.keys():

            self.names_list.append(str(key))

            if self.type in ["Input Indices", "Parameters Indices"]:

                self.index_list.append(data[key][0])
                self.additional_info_list.append(data[key][1])

            else:

                self.index_list.append(len(data[key]["variables"]))

        self.row_count = len(self.names_list)

    def rowCount(self, parent=QModelIndex()):
        return self.row_count

    def columnCount(self, parent=QModelIndex()):
        return self.column_count

    # noinspection PyMethodOverriding
    def headerData(self, section, orientation, role):

        if role == Qt.DisplayRole:

            if orientation == Qt.Horizontal:

                if self.type == "Input Indices":
                    name_tuple = ("Index Name", "Index Reference", "Multiple Input")

                elif self.type == "Parameters Indices":
                    name_tuple = ("Index Name", "Index Reference", "Related Option")

                else:
                    name_tuple = ("Name", "n° Connections", "")

                return name_tuple[section]

            else:
                return "{}".format(section)

        else:

            return None

    def data(self, index, role=Qt.DisplayRole):

        column = index.column()
        row = index.row()

        if role == Qt.DisplayRole:

            if column == 0:
                return self.names_list[row]

            elif column == 1:
                return self.index_list[row]

            elif column == 2:
                return str(self.additional_info_list[row])

        elif role == Qt.BackgroundRole:
            return QColor(Qt.white)

        elif role == Qt.TextAlignmentRole:

            if column == 0:
                return Qt.AlignLeft | Qt.AlignVCenter

            elif column == 1:
                return Qt.AlignCenter

            else:

                if self.type == "Input Indices":
                    return Qt.AlignCenter

                elif self.type == "Parameters Indices":
                    return Qt.AlignLeft | Qt.AlignVCenter

                else:
                    return Qt.AlignCenter

        elif role == Qt.CheckStateRole:

            if index.column() == 2 and self.type == "Input Indices":

                if self.additional_info_list[row]:
                    return Qt.Checked

                else:
                    return Qt.Unchecked

        return None

    def flags(self, index):

        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def context_action_activation(self, row, action):

        if self.type in ["Input Indices", "Parameters Indices"]:

            return False

        else:

            return True

    # <------------ MENU ON CLICK METHODS ------------>

    def onAddClicked(self, row):

        if self.type in ["Input Indices", "Parameters Indices"]:

            pass

        else:

            pass

    def onModifyClicked(self, row):

        if self.type in ["Input Indices", "Parameters Indices"]:

            pass

        else:

            pass

    def onDeleteClicked(self, row):

        if self.type in ["Input Indices", "Parameters Indices"]:

            pass

        else:

            pass



import unittest, traceback, os
from EEETools import costants
from EEETools.Tools.EESCodeGenerator.EES_parser import EESCodeAnalyzer


class EESCodeTestCase(unittest.TestCase):

    def test_parser(self):

        self.results = list()
        self.code_analyzer = EESCodeAnalyzer()
        self.error_found = False

        resource_file_path = os.path.join(costants.TEST_RES_DIR, "TXTFiles", "EESTestScripts")

        self.__test_file(resource_file_path)

        variable_list = list()
        for result in self.results:

            if not result is None:
                result.append_variable_to_list(variable_list)
                print(result.rich_text)

        self.assertFalse(self.error_found)
        self.assertEqual(13, len(variable_list))

    def test_splitter(self):

        self.code_analyzer = EESCodeAnalyzer()
        self.results = list()
        self.error_found = False

        resource_file_path = os.path.join(costants.TEST_RES_DIR, "TXTFiles", "EESSplitterTestScript")

        self.__test_file(resource_file_path, test_split=True)
        self.assertFalse(self.error_found)
        self.assertEqual(3, len(self.results))

    def __test_file(self, file_path, test_split=False):

        resource_file = open(file_path)

        if test_split:

            text = resource_file.read()

            try:

                self.results.extend(self.code_analyzer.get_code_parts(text, print_token=True))

            except:

                traceback.print_exc()
                self.error_found = True

        else:

            lines = resource_file.readlines()

            for line in lines:

                line = line.strip('\n')

                try:

                    self.results.append(self.code_analyzer.parse_string(line, print_token=True))

                except:

                    traceback.print_exc()
                    self.error_found = True

                line = resource_file.readline()


if __name__ == '__main__':
    unittest.main()



import inspect

called_from_setup = False

for frame in inspect.stack():

    if "setup.py" in frame.filename:

        called_from_setup = True
        break

if not called_from_setup:

    from EEETools.Tools.API.terminal_api import paste_default_excel_file, paste_components_documentation, paste_user_manual
    from EEETools.Tools.API.terminal_api import calculate, launch_connection_debug, launch_network_display

else:

    try:

        from EEETools.Tools.Other.resource_downloader import update_resource_folder
        update_resource_folder(force_update=True)

    except:

        pass


from EEETools.Tools.GUIElements.connection_and_block_check import CheckConnectionWidget
from EEETools.Tools.GUIElements.net_plot_modules import display_network
from EEETools.Tools.API.ExcelAPI.modules_importer import import_excel_input
from tkinter import filedialog
import tkinter as tk
import unittest


class MyTestCase(unittest.TestCase):

    def test_block_connection(self):

        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

        array_handler = import_excel_input(excel_path)
        CheckConnectionWidget.launch(array_handler)

        self.assertEqual(True, True)

    def test_network(self):

        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

        array_handler = import_excel_input(excel_path)
        display_network(array_handler)

        self.assertEqual(True, True)

    def test_network_product_fuel(self):

        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

        array_handler = import_excel_input(excel_path)

        try:

            array_handler.calculate()

        except:

            pass

        display_network(array_handler.pf_diagram)

        self.assertEqual(True, True)


if __name__ == '__main__':

    unittest.main()



from EEETools.MainModules import Block
from EEETools.MainModules.support_blocks import Drawer
import xml.etree.ElementTree as ETree
from EEETools import costants


class HeatExchanger(Block):

    def __init__(self, inputID, main_class):

        super().__init__(inputID, main_class)

        self.type = "heat exchanger"
        self.has_support_block = True

        self.support_block.append(Drawer(main_class, self, is_input=True, allow_multiple_input=False))
        self.support_block.append(Drawer(main_class, self, is_input=False, allow_multiple_input=False))

    def add_new_drawer(self, is_input):
        self.support_block.append(Drawer(self.main_class, self, is_input=is_input))

    def is_ready_for_calculation(self):

        for supp_block in self.support_block:

            if not supp_block.is_ready_for_calculation:
                return False

        return True

    def initialize_connection_list(self, input_list):

        if str(input_list[0]) in ["Heat Exchanger", "Scambiatore"]:

            new_conn_input_product = self.main_class.find_connection_by_index(abs(input_list[1]))
            new_conn_output_product = self.main_class.find_connection_by_index(abs(input_list[2]))
            new_conn_input_fuel = self.main_class.find_connection_by_index(abs(input_list[3]))
            new_conn_output_fuel = self.main_class.find_connection_by_index(abs(input_list[4]))

            self.add_connection(new_conn_input_product, is_input=True, append_to_support_block=1)
            self.add_connection(new_conn_output_product, is_input=False, append_to_support_block=1)
            self.add_connection(new_conn_input_fuel, is_input=True, append_to_support_block=0)
            self.add_connection(new_conn_output_fuel, is_input=False, append_to_support_block=0)

        elif str(input_list[0]) in ["Heat Exchanger - Multi Fuel", "Scambiatore - Multi Fuel"]:

            new_conn_input_product = self.main_class.find_connection_by_index(abs(input_list[1]))
            new_conn_output_product = self.main_class.find_connection_by_index(abs(input_list[2]))

            self.add_connection(new_conn_input_product, is_input=True, append_to_support_block=1)
            self.add_connection(new_conn_output_product, is_input=False, append_to_support_block=1)

            for elem in input_list[3:]:

                new_conn = self.main_class.find_connection_by_index(abs(elem))

                if not new_conn is None:
                    is_input = (elem > 0)
                    self.add_connection(new_conn, is_input=is_input, append_to_support_block=0)

        else:

            new_conn_input_fuel = self.main_class.find_connection_by_index(abs(input_list[1]))
            new_conn_output_fuel = self.main_class.find_connection_by_index(abs(input_list[2]))

            self.add_connection(new_conn_input_fuel, is_input=True, append_to_support_block=0)
            self.add_connection(new_conn_output_fuel, is_input=False, append_to_support_block=0)

            for elem in input_list[3:]:

                new_conn = self.main_class.find_connection_by_index(abs(elem))

                if not new_conn is None:
                    is_input = (elem > 0)
                    self.add_connection(new_conn, is_input=is_input, append_to_support_block=1)

    def export_xml_connection_list(self) -> ETree.Element:

        xml_connection_list = ETree.Element("Connections")

        fuels_connections = ETree.SubElement(xml_connection_list, "FuelsConnections")
        product_connections = ETree.SubElement(xml_connection_list, "ProductConnections")

        for support_block in self.support_block:

            if support_block.is_input:

                main_tree = ETree.SubElement(fuels_connections, "Block")

            else:

                main_tree = ETree.SubElement(product_connections, "Block")

            for input_connection in support_block.external_input_connections:

                input_xml = ETree.SubElement(main_tree, "input")
                input_xml.set("index", str(input_connection.index))

            for output_connection in support_block.external_output_connections:

                output_xml = ETree.SubElement(main_tree, "output")
                output_xml.set("index", str(output_connection.index))

        return xml_connection_list

    def append_xml_connection_list(self, input_list: ETree.Element):

        fuels_connections = input_list.find("FuelsConnections")
        product_connections = input_list.find("ProductConnections")

        self.__add_support_blocks(len(fuels_connections.findall("Block")), True)
        self.__add_support_blocks(len(product_connections.findall("Block")), False)

        i = 0
        support_block_array = self.input_support_block
        for connection in fuels_connections.findall("Block"):
            self.__add_connection_by_index(connection, "input", append_to_support_block=support_block_array[i])
            self.__add_connection_by_index(connection, "output", append_to_support_block=support_block_array[i])
            i = i + 1

        i = 0
        support_block_array = self.output_support_block
        for connection in product_connections.findall("Block"):
            self.__add_connection_by_index(connection, "input", append_to_support_block=support_block_array[i])
            self.__add_connection_by_index(connection, "output", append_to_support_block=support_block_array[i])
            i = i + 1

    def __add_connection_by_index(self, input_list: ETree.Element, connection_name, append_to_support_block=None):

        if connection_name == "input":

            is_input = True

        else:

            is_input = False

        for connection in input_list.findall(connection_name):

            new_conn = self.main_class.find_connection_by_index(float(connection.get("index")))

            if new_conn is not None:
                self.add_connection(new_conn, is_input, append_to_support_block=append_to_support_block)

    def __add_support_blocks(self, n_support_blocks, is_input):

        for i in range(1, n_support_blocks):
            self.add_new_drawer(is_input)

    @property
    def input_support_block(self) -> list:

        return_list = list()

        for support_block in self.support_block:

            if support_block.is_input:
                return_list.append(support_block)

        return return_list

    @property
    def output_support_block(self) -> list:

        return_list = list()

        for support_block in self.support_block:

            if not support_block.is_input:
                return_list.append(support_block)

        return return_list

    @classmethod
    def return_EES_needed_index(cls):

        return_dict = {"input_1": [1, False],
                       "output_1": [1, False],
                       "input_2": [1, False],
                       "output_2": [1, False]}

        return return_dict

    @classmethod
    def return_EES_base_equations(cls):

        return_element = dict()

        variables_list = [{"variable": "input_1", "type": costants.ZONE_TYPE_FLOW_RATE},
                          {"variable": "output_1", "type": costants.ZONE_TYPE_FLOW_RATE}]

        return_element.update({"mass_continuity_1": {"variables": variables_list, "related_option": "none"}})

        variables_list = [{"variable": "input_2", "type": costants.ZONE_TYPE_FLOW_RATE},
                          {"variable": "output_2", "type": costants.ZONE_TYPE_FLOW_RATE}]

        return_element.update({"mass_continuity_2": {"variables": variables_list, "related_option": "none"}})

        variables_list = [{"variable": "input_1", "type": costants.ZONE_TYPE_PRESSURE},
                          {"variable": "output_1", "type": costants.ZONE_TYPE_PRESSURE}]

        return_element.update({"pressure_continuity_1": {"variables": variables_list, "related_option": "none"}})

        variables_list = [{"variable": "input_2", "type": costants.ZONE_TYPE_PRESSURE},
                          {"variable": "output_2", "type": costants.ZONE_TYPE_PRESSURE}]

        return_element.update({"pressure_continuity_2": {"variables": variables_list, "related_option": "none"}})

        return return_element

    def return_other_zone_connections(self, zone_type, input_connection):

        connected_drawer = None
        for drawer in self.support_block:

            if drawer.connection_is_in_connections_list(input_connection):
                connected_drawer = drawer
                break

        if zone_type == costants.ZONE_TYPE_FLOW_RATE:

            # In an heat exchanger the flow rate is preserved for each drawer, hence the program identify the drawer
            # to which "input_connection" stream is connected and returns each fluid stream connected to that block

            if connected_drawer is not None:

                return connected_drawer.get_fluid_stream_connections()

            else:

                return list()

        elif zone_type == costants.ZONE_TYPE_FLUID:

            # In an heat exchanger fluid type is preserved for each drawer, hence the program identify the drawer to
            # which "input_connection" stream is connected and returns each fluid stream connected to that block

            if connected_drawer is not None:

                return connected_drawer.get_fluid_stream_connections()

            else:

                return list()

        elif zone_type == costants.ZONE_TYPE_PRESSURE:

            # In an heat exchanger pressure is preserved for each drawer, hence the program identify the drawer to which
            # "input_connection" stream is connected and returns each fluid stream connected to that block

            if connected_drawer is not None:

                return connected_drawer.get_fluid_stream_connections()

            else:

                return list()

        else:

            return list()


import tkinter as tk
import unittest
from tkinter import filedialog

from EEETools import costants
from EEETools.Tools.API.DatAPI.modules_importer import import_dat
from EEETools.Tools.API.ExcelAPI.modules_importer import *


def import_matlab_result(excel_path):

    try:

        result_data = pandas.read_excel(excel_path, sheet_name="Eff Out").values
        return result_data[1, 6]

    except:

        return -100


class ImportTestCase(unittest.TestCase):

    def test_excel_import(self):

        array_handler_list = list()
        resource_excel_path = os.path.join(costants.TEST_RES_DIR, "ImportTestResources", "ExcelTestFiles")

        i = 1
        excel_path = os.path.join(resource_excel_path, "Sample Excel Input " + str(i) + ".xlsm")

        while os.path.isfile(excel_path):

            array_handler = import_excel_input(excel_path)
            result = import_matlab_result(excel_path)

            if result != -100:

                array_handler.options.calculate_on_pf_diagram = False
                array_handler.calculate()

                print(array_handler)

                array_handler_list.append(array_handler)
                useful_effect = array_handler.useful_effect_connections[0]

                self.assertEqual(round(result, 6), round(useful_effect.rel_cost, 6))

            i += 1
            excel_path = os.path.join(resource_excel_path, "Sample Excel Input " + str(i) + ".xlsm")

    def test_excel_direct_calculation(self):

        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()
        calculate_excel(excel_path)

        self.assertTrue(True)

    def test_dat_import_export(self):

        array_handler_list = list()
        resource_excel_path = os.path.join(costants.TEST_RES_DIR, "ImportTestResources", "ExcelTestFiles")
        resource_dat_path = os.path.join(costants.TEST_RES_DIR, "ImportTestResources", "DatTestFiles")

        i = 1
        excel_path = os.path.join(resource_excel_path, "Sample Excel Input " + str(i) + ".xlsm")

        while os.path.isfile(excel_path):

            array_handler = import_excel_input(excel_path)
            array_handler.calculate()
            result_excel = array_handler.useful_effect_connections

            dat_path = os.path.join(resource_dat_path, "Sample Dat " + str(i) + ".dat")
            export_dat(dat_path, array_handler)
            array_handler_dat = import_dat(dat_path)
            array_handler_dat.options.calculate_on_pf_diagram = False
            array_handler_list.append(array_handler_dat)

            array_handler_dat.calculate()
            result_dat = array_handler_dat.useful_effect_connections

            difference = 0
            sum_exergy = 0

            for result in result_dat:

                difference += result.rel_cost*result.exergy_value
                sum_exergy += result.rel_cost*result.exergy_value

            for result in result_excel:

                difference -= result.rel_cost * result.exergy_value
                sum_exergy += result.rel_cost * result.exergy_value

            err = 2*difference/sum_exergy

            self.assertEqual(round(err, 7), 0)

            i += 1
            excel_path = os.path.join(resource_excel_path, "Sample Excel Input " + str(i) + ".xlsm")

    def test_download_link(self):

        from EEETools.Tools.Other.fernet_handler import FernetHandler
        FernetHandler()

        self.assertTrue(True)

    def test_excel_calculate(self):

        import EEETools
        EEETools.calculate()
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



from EEETools.version import VERSION
from setuptools import setup
from os import path, listdir

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def __get_install_requires():

    install_requires = [

        'future~=0.18.2',
        'PyQt5~=5.15.4',
        'pandas~=1.2.3',
        'openpyxl~=3.0.7',
        'cryptography~=3.4.6',
        'pyvis~=0.1.9',
        'rply~=0.7.8',
        'setuptools~=54.2.0',
        'xlrd~=1.2.0',
        'numpy',
        'requests'

    ]

    import platform

    if platform.system() == "Windows":
        install_requires.append('pywin32')

    return install_requires


def __get_packages() -> list:

    ROOT_DIR = path.join(path.dirname(__file__), "EEETools")
    TEST_DIR = path.join(path.dirname(__file__), "test")
    packages = append_sub_dir(ROOT_DIR, list())
    packages = append_sub_dir(TEST_DIR, packages)
    return packages


def append_sub_dir(element_path, input_list: list, parent_name=None) -> list:

    if path.isdir(element_path):

        name = path.basename(element_path)

        if "__" not in name:

            if parent_name is not None:
                name = "{}.{}".format(parent_name, name)

            input_list.append(name)

            for sub_name in listdir(element_path):
                input_list = append_sub_dir(path.join(element_path, sub_name), input_list, parent_name=name)

    return input_list


setup(

    name='3ETool',
    version=VERSION,
    license='GNU GPLv3',

    author='Pietro Ungar',
    author_email='pietro.ungar@unifi.it',

    description='Tools for performing exergo-economic and exergo-environmental analysis',
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://tinyurl.com/SERG-3ETool',
    download_url='https://github.com/SERGGroup/3ETool/archive/refs/tags/{}.tar.gz'.format(VERSION),

    project_urls={

        'Documentation': 'https://drive.google.com/file/d/1Dzdz4_EAKyY9c8nOm2SKTIqSXfV74w2-/view?usp=sharing',
        'Source': 'https://github.com/SERGGroup/3ETool',
        'Tracker': 'https://github.com/SERGGroup/3ETool/issues',

    },

    packages=__get_packages(),

    install_requires=__get_install_requires(),

    classifiers=[

        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

    ]

)



from EEETools.MainModules.main_module import CalculationOptions
from EEETools.Tools.API.DatAPI.modules_importer import export_dat
from EEETools.Tools.API.Tools.main_tools import get_result_data_frames
from openpyxl import Workbook, load_workbook, styles, utils
from EEETools.MainModules.main_module import ArrayHandler
from datetime import date, datetime
import math, pandas, os


def calculate_excel(excel_path, calculation_option=None):

    array_handler = import_excel_input(excel_path)
    array_handler.calculate()

    if calculation_option is not None and type(calculation_option) == CalculationOptions:

        array_handler.options = calculation_option

    export_solution_to_excel(excel_path, array_handler)


def convert_excel_to_dat(excel_path: str):

    array_handler = import_excel_input(excel_path)

    if ".xlsm" in excel_path:

        dat_path = excel_path.replace(".xlsm", ".dat")

    elif ".xlsx" in excel_path:

        dat_path = excel_path.replace(".xlsx", ".dat")

    else:

        dat_path = excel_path.replace(".xls", ".dat")

    export_dat(dat_path, array_handler)


def import_excel_input(excel_path) -> ArrayHandler:

    __check_excel_version(excel_path)
    array_handler = ArrayHandler()

    # import connections
    excel_connection_data = pandas.read_excel(excel_path, sheet_name="Stream")

    for line in excel_connection_data.values:

        line = line.tolist()
        if not math.isnan(line[0]):
            new_conn = array_handler.append_connection()

            new_conn.index = line[0]
            new_conn.name = str(line[1])
            new_conn.exergy_value = line[2]

    # import blocks
    excel_block_data = pandas.read_excel(excel_path, sheet_name="Componenti")

    for line in excel_block_data.values:

        line = line.tolist()

        if not (math.isnan(line[0]) or type(line[0]) is str):

            if line[0] > 0:

                if "Heat Exchanger" in str(line[2]) or "Scambiatore" in str(line[2]):

                    new_block = array_handler.append_block("Heat Exchanger")
                    excel_connection_list = list()
                    excel_connection_list.append(str(line[2]))
                    excel_connection_list.extend(line[5:])

                else:

                    new_block = array_handler.append_block(str(line[2]))
                    excel_connection_list = line[5:]

                new_block.index = line[0]
                new_block.name = str(line[1])
                new_block.comp_cost = line[3]

                new_block.initialize_connection_list(excel_connection_list)

            else:

                array_handler.append_excel_costs_and_useful_output(line[5:-1], line[0] == 0, line[3])

    return array_handler


def export_solution_to_excel(excel_path, array_handler: ArrayHandler):

    result_df = get_result_data_frames(array_handler)

    # generation of time stamps for excel sheet name
    today = date.today()
    now = datetime.now()
    today_str = today.strftime("%d %b")
    now_str = now.strftime("%H.%M")

    for key in result_df.keys():

        __write_excel_file(excel_path, sheet_name=(key + " - " + today_str + " - " + now_str),
                           data_frame=result_df[key])


def __write_excel_file(excel_path, sheet_name, data_frame: dict):
    data_frame = __convert_result_data_frames(data_frame)

    if not os.path.isfile(excel_path):

        wb = Workbook()

    else:

        wb = load_workbook(excel_path)

    if not sheet_name in wb.sheetnames:
        wb.create_sheet(sheet_name)

    sheet = wb[sheet_name]

    col = 2
    for key in data_frame.keys():

        row = 2
        sub_data_frame = data_frame[key]
        n_sub_element = len(sub_data_frame["unit"])

        if key == "Name":

            column_dimension = 35

        elif key == "Stream":

            column_dimension = 8

        else:

            column_dimension = 20

        if n_sub_element == 0:

            sheet.merge_cells(start_row=row, start_column=col, end_row=row + 1, end_column=col)
            n_sub_element = 1

        else:

            if n_sub_element > 1:
                sheet.merge_cells(start_row=row, start_column=col, end_row=row, end_column=col + n_sub_element - 1)

            for n in range(n_sub_element):
                row = 3
                cell = sheet.cell(row, col + n, value="[" + sub_data_frame["unit"][n] + "]")
                cell.alignment = styles.Alignment(horizontal="center", vertical="center")
                cell.font = styles.Font(italic=True, size=10)

        cell = sheet.cell(2, col, value=key)
        cell.alignment = styles.Alignment(horizontal="center", vertical="center")
        cell.font = styles.Font(bold=True)

        for n in range(n_sub_element):

            row = 5
            data_list = (sub_data_frame["values"])[n][0]
            sheet.column_dimensions[utils.get_column_letter(col)].width = column_dimension

            for data in data_list:
                sheet.cell(row, col, value=data)
                row += 1

            col += 1

    wb.save(excel_path)


def __convert_result_data_frames(data_frame: dict) -> dict:

    new_data_frame = dict()

    for key in data_frame.keys():

        sub_dict = __get_sub_dict(key)
        sub_dict.update({"values": list()})

        if not sub_dict["name"] in new_data_frame.keys():

            new_data_frame.update({sub_dict["name"]: sub_dict})

        else:

            (new_data_frame[sub_dict["name"]])["unit"].append(sub_dict["unit"][0])

        (new_data_frame[sub_dict["name"]])["values"].append([data_frame[key]])

    return new_data_frame


def __get_sub_dict(key):

    if "[" in key:

        parts = key.split("[")

        name = parts[0].replace("[", "")
        measure_unit = [parts[1].split("]")[0].replace("]", "")]

    else:

        name = key
        measure_unit = list()

    return {"name": name, "unit": measure_unit}


def __check_excel_version(excel_path):

    pass